# -*- coding: utf-8 -*-
"""
Created on 2018/9/20

@author: gaoan
"""

P_TIGER_ID = "tiger_id"
P_METHOD = "method"
P_CHARSET = "charset"
P_SIGN_TYPE = "sign_type"
P_SIGN = "sign"
P_TIMESTAMP = "timestamp"
P_VERSION = "version"
P_NOTIFY_URL = "notify_url"
P_DEVICE_ID = "device_id"

COMMON_PARAM_KEYS = {P_TIGER_ID, P_METHOD, P_CHARSET, P_SIGN_TYPE, P_SIGN, P_TIMESTAMP, P_VERSION, P_NOTIFY_URL,
                     P_DEVICE_ID}
P_BIZ_CONTENT = "biz_content"
